## rucdr available here https://github.com/antpiron/rucdr/
library(rucdr)
library(RedRibbon)
library(data.table)
library(clusterProfiler)
library(gprofiler2)
library(functional)
library(magrittr)
library(stringr)
library(ggpubr)
library(ggVennDiagram)
library(ggplotify)
library(DiagrammeR)
library(ggtext)
library(glue)
library(RRHO2)
library(RRHO)


setwd("~/git/bioinfo/ph.d/manuscripts/RedRibbon")

import <- function (filename, paths = c())
{
    if ( isAbsolutePath(filename) )
    {
        source(filename)
        return()
    }

    wd <- tryCatch({dirname(sys.frame(1)$ofile)},
                   error=function (e) {file.path(".")})
    path <- file.path(wd, filename)
    
    if (file.exists(path) )
    {
        source(path)
        return()
    }
        
    paths <- c(paths, strsplit(Sys.getenv("R_IMPORT_DIR"), ':')[[1]])
    for ( cpath in paths )
    {
        path <- file.path(cpath, filename)
        if (file.exists(path) )
        {
                    source(path)
                    return()
        }
    }
    stop(paste("Unable to find:", filename))
}

PATHS=c(".", "~/git/bioinfo/ph.d/manuscripts/RedRibbon", "src", "/srv/projects/tiger/src/r-lib/", "/srv/projects/tiger.bak/src/r-lib/", "~/git/bioinfo/ph.d/projects/tiger/src/r-lib")
Sys.setenv(R_IMPORT_DIR = paste(PATHS, collapse=':'))

import("perfect.R")
import("gtf.R")
import("performance.R")
import("rrho.R")
import("txdb.R")
import("cache.R")
import("density.R")
import("accuracy.R")
import("originalRRHO-wrapper.R")


version_dir <- "01-review-bioinformatics"
output_dir <- file.path("output", version_dir)
figures_dir  <- file.path(output_dir, 'figures')
dir.create(figures_dir, showWarnings=FALSE, recursive=T)
tsv_dir <- file.path(output_dir, 'tsv')
dir.create(tsv_dir, showWarnings=FALSE, recursive=T)

srsf6_dir <- file.path(figures_dir, "srp55")
dir.create(srsf6_dir, showWarnings=FALSE, recursive=T)

gtf <- newGTF("input/gtf/gencode.v37.annotation.gtf.gz")
t2g <- tx2gene()

###################################
## PATHWAYS
TYPE_2_DIABETES <- c("SRSF6", "IKBKB", "ABCC8", "CACNA1C", "PRKCD", "MAPK8", "CACNA1D", "MAPK9", "MAPK3", "PDX1", "CACNA1B", "INSR",
                     "MAPK10", "GCK", "SLC2A2", "INS", "HKDC1", "PKM")

POSITIVE_REGULATION_APOPTOSIS <- c("SRSF6", "BAX", "BCL2L11", "DIABLO", "ITGB3BP", "BOK", "BCLAF1", "APBB1", "ING4", "SEPT4", "CDKN2A",
                                   "FOXO1", "GADD45A", "CSRNP3", "RBM5", "PIDD", "MAP2K6", "MAP3K10", "TIAM1", "APAF1", "MTCH1",
                                   "MELK", "NR4A1")

INSULIN_SECRETION <- c("SRSF6", "ADCY6", "ADCY2", "CAMK2B", "CREB3L3", "ADCY8", "ABCC8", "CACNA1C", "CACNA1D", "PDX1", "RYR2", "CHRM3", "PLCB4",
                       "FXYD2", "GLP1R", "INS", "GNAS", "ATF2", "RIMS2", "CREB1", "CREB3L2", "GCK", "SLC2A2", "SNAP25")

JNK_SIGNALING_CASCADE <- c("SRSF6", "MINK1", "MAPK10", "ROR2", "FGF12", "MAP3K11", "MAP4K2", "MAPK4K3", "MAPK9", "MAPK8", "NOD1", "IRAK1", "MAP3K7", "RPS27A", "MAP3K10", "MYD88")
SRSF6_REGULATION <- unique(c(TYPE_2_DIABETES, POSITIVE_REGULATION_APOPTOSIS, INSULIN_SECRETION, JNK_SIGNALING_CASCADE))


MY_PATHWAYS <- list(UCDR_TYPE_2_DIABETES = TYPE_2_DIABETES,
                    UCDR_POSITIVE_REGULATION_APOPTOSIS = POSITIVE_REGULATION_APOPTOSIS,
                    UCDR_INSULIN_SECRETION = INSULIN_SECRETION,
                    UCDR_JNK_SIGNALING_CASCADE = JNK_SIGNALING_CASCADE,
                    UCDR_SRSF6_REGULATION = SRSF6_REGULATION)

###################################
## Perfect plots
gg_perfect <- perfect()
ggsave(file.path(figures_dir, "fig-x-rrho-perfect.png"), gg_perfect, dpi=600, width=12, height=10)


###################################
## Performance plots

## for ( size_t n = (1llu << 10)  ; n < (1llu << 18) + 1 ; n = n << 1)
orig.perf.fn <- file.path(tsv_dir, "p-rrho-original.tsv")
if (! file.exists(orig.perf.fn) )
{
    orig.perf <- do.call(rbind,
                         lapply(10:16,
                                function (expo)
                                {
                                    list_length  <- 2^expo
                                    step <- floor(sqrt(list_length) )
                                    #
                                    cat(paste0(list_length,"\n"))
                                    data <- 1:list_length
                                    names(data)  <- data
                                    #
                                    start_time <- Sys.time()
                                    rucdr::RRHO(data, data)
                                    end_time <- Sys.time()
                                    #
                                    diff_time <- difftime(end_time, start_time, units = "secs")
                                    cat(paste0(diff_time,"\n"))
                                    #
                                    return( data.table(n=list_length, step=step, time=as.double(diff_time),
                                                       rrho.per.sec=list_length*list_length / as.double(diff_time) / step / step) )
                                }))
    fwrite(orig.perf, orig.perf.fn, sep='\t')
} else orig.perf <- fread(orig.perf.fn)

bench <- performance(rrho.original.fn = orig.perf.fn)
## ggsave(file.path(figures_dir, "bench-redribbon.png"),  bench$redribbon.rrho,  dpi=300, height = 4, width = 4)
## ggsave(file.path(figures_dir, "bench-cmp.png"),  bench$comparison,  dpi=300, height = 4, width = 4)
## ggsave(file.path(figures_dir, "bench-perm.png"),  bench$redribbon.perm,  dpi=300, height = 4, width = 4)
ggsave(file.path(figures_dir, "fig-x-performance.png"), bench$combined,  dpi=600, height = 5, width = 12)


###################################
## Type 1 error control (padj) for permutation

type1_n <- 1000
aaa <- sample(type1_n)

type1.fn <- file.path(tsv_dir, "type1_error.tsv")
type1 <- do.call(rbind,
                 lapply(1:1000,
                        function (i)
                        {
                            type1.dt <- data.table(id = 1:type1_n, a = aaa, b =  sample(type1_n))
                            type1.rr <- RedRibbon(type1.dt)
                            type1.quad <- quadrants(type1.rr, algorithm = "ea",
                                                    permutation = TRUE,
                                                    niter = 4*96)
                            ggRedRibbon(type1.rr, quadrants=type1.quad)
                            data.table(padj = type1.quad$whole$padj, pvalue = type1.quad$whole$pvalue)
                        }))
fwrite(type1, type1.fn, sep='\t')


type1.stat.fn <- file.path(tsv_dir, "type1_error_stat.tsv")

type1.dt <- data.table(freq_0.05 = sum(type1$padj <= 0.05) / type1_n,
                       stat = sum(type1$padj <= 0.05),
                       n = type1_n,
                       pval = pbinom(sum(type1$padj <= 0.05), type1_n, 0.05))
fwrite(type1.dt, type1.stat.fn, sep='\t')


type1.gg <- ggplot2::ggplot(type1,
                     aes(x=padj)) + 
    geom_density()
ggsave(file.path(figures_dir, "fig-x-type1-error-distribution.png"), type1.gg,  dpi=600, height = 4.5, width = 12)


##################
## Accuracy

## PG timecourse

pg_enrichment <- function (self, directions="upup",
                           msigdb.fn = "/srv/datasets/public/references/msigdb/v7.0/GMTs/c2.cp.v7.0.symbols.gmt",
                           set.universe=FALSE)
{
    
    if (! file.exists(msigdb.fn) )
        msigdb.fn <- "/srv/genomic_data/msigdb/v7.0/GMTs/c2.cp.v7.0.symbols.gmt"
    msigdb <- read.gmt(msigdb.fn)
    
    genes <- unique(unlist(sapply(directions, function (direction) self$ab[self$quad[[direction]]$positions,]$shortid)))
    genes.ensg <- unique(unlist(sapply(directions, function (direction) self$ab[self$quad[[direction]]$positions,]$ensg)))
    
    universe  <- NULL
    if (set.universe)
        universe <- unique(self$ab$shortid)

    db <- as.data.table(msigdb)
    names(db) <- c("term", "gene")
    TERM2GENE <- db
    enrich.clusterprofiler <-  enricher(genes, universe=universe, TERM2GENE=TERM2GENE)

    name <- paste0(directions, collapse="_")
    if ( is.null(self$enrichment) )
        self$enrichment <- list()

    sources <- c("GO:BP",
                 "KEGG",  "REAC",
                 "CORUM",
                 "WP")
    enrich.gprofiler <- gost(genes.ensg, organism = "hsapiens", sources=sources)

    
    self$enrichment[[name]]  <- structure(
        list(clusterprofiler = enrich.clusterprofiler,
             gprofiler = as.data.table(enrich.gprofiler$result)),
        class = "enrichment"
    )
    return(self)
}

fread_de <- function (fn)
{
    res <- fread(fn, sep="\t")
    colnames(res)[1]  <- "ensg"
    setorder(res, "ensg")
    return(res)
}

cmp_rrho_redribbon <- function (a_de, b_de, expression_b = NULL, labels = c("a", "b"))
{
    df <- merge(a_de[, c("ensg", "log2FoldChange")],
                b_de[, c("ensg", "log2FoldChange")],
                by="ensg")
    colnames(df)  <- c("ensg", "a", "b")
    df  <- merge(df, gtf$data[feature == "gene", c("ensg", "shortid")], by = "ensg", all.x = TRUE)
    setorder(df, "ensg")

    ## Old RRHO method
    ## Published with Lorella/Piero
    lista <- df$a
    listb <- df$b
    names(lista) <- names(listb) <- df$ensg

    cat("Running old RRHO\n")
    rrho_old <- rucdr::RRHO(lista, listb)
    cat("Running old RRHO: getting ovelap\n")
    rrho_quadrants  <- list(
        upup = data.table(ensg = getUPUP(rrho_old)),
        downup = data.table(ensg = getDOWNUP(rrho_old)),
        downdown = data.table(ensg = getDOWNDOWN(rrho_old)),
        updown = data.table(ensg = getUPDOWN(rrho_old)))

    ## gprofiler original PG enrich
    cat("Running old RRHO: enrichment\n")
    sources <- c("GO:BP",
                 "KEGG",  "REAC",
                 "CORUM",
                 "WP")
    rrho_ernrichment <- list(
        upup = as.data.table(gost(rrho_quadrants$upup$ensg, organism = "hsapiens", sources=sources)$result),
        downup = as.data.table(gost(rrho_quadrants$downup$ensg, organism = "hsapiens", sources=sources)$result),
        downdown = as.data.table(gost(rrho_quadrants$downdown$ensg, organism = "hsapiens", sources=sources)$result),
        updown = as.data.table(gost(rrho_quadrants$updown$ensg, organism = "hsapiens", sources=sources)$result))

    ## mat <- expression[, .SD, .SDcols = names(expression) %like% "D8PG|D8P22G|D8CG|D4PG|D4P22G|D4CG"]
    fc.obj <- if ( is.null(expression_b) ) NULL else {
                                                    cat("Compute correlation")
                                                    mat <- expression_b[ensg %in% df$ensg,]
                                                    if ( nrow(mat) != nrow(df) )
                                                        stop("nrow(mat) != nrow(df)")
                                                    
                                                    if ( any(mat$ensg != df$ensg) )
                                                        stop("any(mat$ensg != df$ensg)")
                                                    
                                                    setorder(mat, "ensg")
                                                    correlation <- rrho_expression_prediction(mat)
                                                    newFC(correlation$index, correlation$beta[,2])
                                                }
    
    cat("Running RedRibbon\n")
    rr <- RedRibbon(df, enrichment_mode = "hyper-two-tailed", correlation=fc.obj)
    cat("Running RedRibbon: quadrants\n")
    quadrants <- quadrants(rr, algorithm="ea", whole = FALSE, permutation = TRUE)
    
    ## gprofiler RedRibbon PG enrich
    cat("Running RedRibbon: enrichment\n")
    enrichment <- list(ab = df, quad = quadrants) %>%
        pg_enrichment(directions=c("upup")) %>%
        pg_enrichment(directions=c("downup")) %>%
        pg_enrichment(directions=c("downdown")) %>%
        pg_enrichment(directions=c("updown")) 
    
    structure(
        list(labels = labels,
             ab = df,
             RedRibbon = list(
                 rr = rr,
                 quadrants = quadrants,
                 enrichment = enrichment
             ),
             RRHO = list(
                 rrho = rrho_old,
                 quadrants = rrho_quadrants,
                 enrichment = rrho_ernrichment
             )),
        class="cmp_rrho_redribbon")
}


ggT2DvsD8PG <- function(self, directions = c("downup", "upup", "updown", "downdown"), ggarrange.labels=c("C", "D"))
{
    quadrants <- self$RedRibbon$quadrants
    enrichment <- self$RedRibbon$enrichment
    df <- self$ab
    rrho_ernrichment <- self$RRHO$enrichment
    rrho_quadrants <- self$RRHO$quadrants
    
    gg.rr <- ggRedRibbon(self$RedRibbon$rr, quadrants=quadrants, base_size = 20,
                    show.quadrants = TRUE, show.pval = TRUE, labels = self$labels)  +
        coord_fixed(ratio = 1, clip = "off") +
        theme(legend.position = c(0.90, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)))
    
    categories <- c("RedRibbon", "Both", "RRHO")
    categories <- factor(categories, levels=rev(categories))

    sets.fun <- function (rr, rrho, direction="up-up", feature="genes")
    {
        values <- c(length(setdiff(rr, rrho)), length(intersect(rr, rrho)), length(setdiff(rrho, rr)))
        cs <- cumsum(values)
        cs  <- c(values[1] / 2, values[2] / 2 + cs[1],  values[3] / 2 + cs[2])
        data.table(nbr=values,tool=categories, direction=direction, feature=feature, sum=cs)
    }

 
    bar.dt <- do.call(rbind,
                      lapply(directions,
                             function (direction)
                             {
                                 direction.str <- gsub("^(down|up)(.*)$",  "\\1-\\2", direction)
                                 rbind(
                                     sets.fun(df[quadrants[[direction]]$positions]$ensg, rrho_quadrants[[direction]]$ensg, direction=direction.str),
                                     sets.fun(enrichment$enrichment[[direction]]$gprofiler$term_id,
                                              rrho_ernrichment[[direction]]$term_id, direction=direction.str, feature="pathways"))
                                     
                             }))
    
    ggbarplot <- function(dt)
    {
        ggplot(data=dt, aes(x=direction, y=nbr,  fill = tool)) + 
            geom_bar(stat="identity")  + labs(fill = "") +
            geom_label_repel(aes(label = nbr, y = sum), label.size = NA, nudge_y = 1, force=0.5, direction="y", show.legend = FALSE) +
            xlab(NULL) + ylab(NULL) + 
            theme_minimal() + theme(axis.text.x = element_text(angle = 20, vjust = 1.2, hjust=0.8),
                                    axis.title.y=element_blank(),
                                    axis.text.y=element_blank(),
                                    axis.ticks.y=element_blank())
    }

    gg.barplot <- ggpubr::ggarrange(ggbarplot(bar.dt[feature=="genes",]) + theme(axis.text.x = element_blank() ) ,
                                    ggbarplot(bar.dt[feature=="pathways",]),
                                    nrow=2, common.legend = TRUE, legend="right") +
        theme(panel.background = element_rect("white", "white", 0, NULL))
    
    list(ggarrange = ggpubr::ggarrange(gg.rr, gg.barplot, ncol=2, widths=c(5,4), labels = ggarrange.labels),
         ggRedRibbon = gg.rr,
         barplot = gg.barplot)
}

resD8PG <- fread_de("input/de-d8pg.tsv")

filename.t2d.full <- "input/de-t2d-full.tsv"
resT2D <- fread_de(filename.t2d.full)

filename.expression <- "input/salmon.genes.tpm.tsv.gz"
expression <- fread(filename.expression, sep="\t")
setnames(expression, "V1", "ensg")
setorder(expression, "ensg")


##
expression_b <- expression[, .SD, .SDcols = names(expression) %like% "D8PG|D8P22G|D8CG|D4PG|D4P22G|D4CG"]

## T2D_D8PG_D4PG <- cmp_rrho_redribbon(resT2D, resD8PG.D4PG.timecourse, expression_b = NULL,
##                                     labels = c("T2D vs CTL", "D8PG vs D4PG timecourse corrected"))
## gg_T2D_D8PG_D4PG <- ggplot(T2D_D8PG_D4PG)
## ggsave(file.path("/tmp", "T2D_D8PG_D4PG.png"), gg_T2D_D8PG_D4PG, dpi=600, height = 6, width = 12) 

T2D_D8PG_fn <- file.path(output_dir, "t2d-d8pg.rds")
if (! file.exists(T2D_D8PG_fn) )
{
    T2D_D8PG <- cmp_rrho_redribbon(resT2D, resD8PG, expression_b = NULL,
                                   labels = c("T2D vs CTL", "D8PG vs CTL"))
                                        # Save an object to a file
    saveRDS(T2D_D8PG, file = T2D_D8PG_fn)
} else
    T2D_D8PG <- readRDS(file = T2D_D8PG_fn)

gg_T2D_D8PG <- ggT2DvsD8PG(T2D_D8PG, ggarrange.labels=c("D", "E"))
ggsave(file.path(figures_dir, "fig-x-T2D-D8PG.png"), gg_T2D_D8PG$ggRedRibbon,  dpi=600, width = 7, height = 5)

ggsave(file.path(figures_dir, "fig-x-T2D-D8PG-barplot.png"), gg_T2D_D8PG$barplot,  dpi=600, width = 5, height = 5)


write.enrich.t2d.d8pg <- function(de, direction, file.prefix="a-b-")
{
    lapply(direction,
           function (dir)
           {
               data  <- de$enrichment[[dir]]
               print(names(data))
               if ( "gprofiler" %in% names(data) )
                   data <- data$gprofiler
               fwrite(as.data.table(data),
                      file.path(tsv_dir,
                                paste0(file.prefix, dir, ".tsv")),
                      sep="\t")
           })
}

write.enrich.t2d.d8pg(T2D_D8PG$RRHO, c("upup", "downdown", "updown", "downup"),
                      file.prefix="T2D-D8PG-RRHO-")
write.enrich.t2d.d8pg(T2D_D8PG$RedRibbon$enrichment,
                      c("upup", "downdown", "updown", "downup"), file.prefix="T2D-D8PG-RedRibbon-")


nrow(T2D_D8PG$ab)
sqrt(nrow(T2D_D8PG$ab))

inter.upup <- merge(T2D_D8PG$RRHO$enrichment$upup, T2D_D8PG$RedRibbon$enrichment$enrichment$upup$gprofiler, by = "term_id")$term_id
inter.upup.only.rr <- T2D_D8PG$RedRibbon$enrichment$enrichment$upup$gprofiler[! term_id %in% inter.upup,]
inter.upup.only.rrho <- T2D_D8PG$RRHO$enrichment$upup[! term_id %in% inter.upup,]

setorder(inter.upup.only.rr, "p_value")

inter.upup.only.rr[, c("term_name", "p_value")]

fwrite(inter.upup.only.rr[, c("term_name", "p_value")],
       file.path(tsv_dir, "T2D_D8PG_RedRibbon_Only_UpUp.tsv"),
       sep = "\t")

inter.updown <- merge(T2D_D8PG$RRHO$enrichment$updown, T2D_D8PG$RedRibbon$enrichment$enrichment$updown$gprofiler, by = "term_id")$term_id
inter.updown.only.rr <- T2D_D8PG$RedRibbon$enrichment$enrichment$updown$gprofiler[! term_id %in% inter.updown,]
inter.updown.only.rrho <- T2D_D8PG$RRHO$enrichment$updown[! term_id %in% inter.updown,]


## Perfect accuracy
## stats.perfect.better.dt <- stats.grid.ea(generate.fn=generate.perfect.better.fc, n.samples=192)

## gg_accuracy_perfect_ea_classic <- ggpubr::ggarrange(gg.cmp(stats.perfect.better.dt, "TPR", algorithms=c("ea", "classic"), x.angle = 20),
##                                                     gg.cmp(stats.perfect.better.dt, "TNR", algorithms=c("ea", "classic"), x.angle = 20),
##                                                     gg.cmp(stats.perfect.better.dt, "ACC", algorithms=c("ea", "classic"), x.angle = 20) + ggtitle("Accuracy"),
##                                                     ncol=3, nrow=1)
## ggsave(file.path(figures_dir, "fig-x-accuracy-perfect-ea-classic.png"), gg_accuracy_perfect_ea_classic,  dpi=600, height = 4, width = 12)

## gg_accuracy_perfect <- ggpubr::ggarrange(gg.cmp(stats.perfect.better.dt, "TPR", x.angle = 35),
##                                          gg.cmp(stats.perfect.better.dt, "TNR", x.angle = 35),
##                                          gg.cmp(stats.perfect.better.dt, "ACC", x.angle = 35) + ggtitle("Accuracy"),
##                                          ncol=3, nrow=1)
## ggsave(file.path(figures_dir, "fig-x-accuracy-perfect.png"), gg_accuracy_perfect,  dpi=600, height = 4, width = 12)


## Correlated accuracy
stats.better.dt <- stats.grid.ea(generate.fn=generate.better.fc, n.samples=192)

gg_accuracy_ea_classic <- ggpubr::ggarrange(gg.cmp(stats.better.dt, "TPR", algorithms=c("ea", "classic"), x.angle = 20),
                                         gg.cmp(stats.better.dt, "TNR", algorithms=c("ea", "classic"), x.angle = 20),
                                         gg.cmp(stats.better.dt, "ACC", algorithms=c("ea", "classic"), x.angle = 20) + ggtitle("Accuracy"),
                                         ncol=3, nrow=1)
ggsave(file.path(figures_dir, "fig-x-accuracy-ea-classic.png"), gg_accuracy_ea_classic,  dpi=600, height = 4, width = 12)

gg_accuracy <- ggpubr::ggarrange(gg.cmp(stats.better.dt, "TPR", x.angle = 35),
                                 gg.cmp(stats.better.dt, "TNR", x.angle = 35),
                                 gg.cmp(stats.better.dt, "ACC", x.angle = 35) + ggtitle("Accuracy"),
                                 ncol=3, nrow=1)
ggsave(file.path(figures_dir, "fig-x-accuracy.png"), gg_accuracy,  dpi=600, height = 4, width = 12)



dt.noisy <- generate.better.fc(n = 23000, n.corr = 5000)
dt.noisy[, gene := 1:23000]
rr.noisy  <- RedRibbon(dt.noisy)
quad.noisy <- quadrants(rr.noisy, whole=FALSE, algorithm = "ea", permutation=FALSE)

gg_artificial_fc <- ggRedRibbon(rr.noisy, quadrants = quad.noisy)  +
        coord_fixed(ratio = 1, clip = "off") +
        theme(legend.position = c(0.9, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)))
ggsave(file.path(figures_dir, "fig-x-artificial-noisy.png"), gg_artificial_fc,  dpi=600, height = 5, width = 6)

dt.perfect <- generate.perfect.better.fc(n = 23000, n.corr = 5000)
dt.perfect[, gene := 1:23000]
rr.perfect  <- RedRibbon(dt.perfect)
quad.perfect <- quadrants(rr.perfect, whole=FALSE, algorithm = "ea", permutation=FALSE)

gg_perfect_artificial_fc <- ggRedRibbon(rr.perfect, quadrants = quad.perfect) +
        coord_fixed(ratio = 1, clip = "off") +
        theme(legend.position = c(0.9, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)))
ggsave(file.path(figures_dir, "fig-x-artificial-perfect.png"), gg_perfect_artificial_fc,  dpi=600, height = 5, width = 6)


## Iterate
iterate.dt <- stats.iterate.stepsize( n.samples = 8*96)
## iterate.dt

gg.accuracy.iterate <- ggpubr::ggarrange(ggIterate(iterate.dt, "TPR"),
                                         ggIterate(iterate.dt, "TNR"),
                                         ggIterate(iterate.dt, "ACC") + ggtitle("Accuracy"),
                                         ncol=3)
## gg.accuracy.iterate
ggsave(file.path(figures_dir, "fig-x-accuracy-iterate.png"), gg.accuracy.iterate,  dpi=600, height = 4, width = 12)


## Orignal RRHO bug
test.dt <- generate.better.fc(23000, 5000)
test.dt[, gene := 1:23000]
RRHO_obj <- originalRRHO(as.data.frame(test.dt[, c("gene", "a")]), as.data.frame(test.dt[, c("gene", "b")]), labels = c("a", "b"))
## file.move(file.path(RRHO_obj$outputdir, "RRHOMapa_VS_b.jpg"),
##          file.path(figures_dir, "RRHO_bug_1.jpg") )

rr_bug <- RedRibbon(test.dt$a, test.dt$b, enrichment_mode = "hyper-two-tailed")
gg_bug <- ggRedRibbon(rr_bug) +
        coord_fixed(ratio = 1, clip = "off") +
        theme(legend.position = c(0.9, 0.5),
              legend.background = element_rect(fill=alpha('white', 0.2)))

## ggsave(file.path(figures_dir, "RRHO_bug_2.png"), gg_bug,  dpi=600, height = 5, width = 6)

library(jpeg)

rrho_jpeg <- readJPEG(file.path(RRHO_obj$outputdir, "RRHOMapa_VS_b.jpg"))
gg_jpeg <- ggplot() + 
    background_image(rrho_jpeg) +
    # This ensures that the image leaves some space at the edges
    theme(plot.margin = margin(t=1, l=1, r=1, b=1, unit = "cm")) +
        coord_fixed(ratio = 1, clip = "off")

gg_bug_panel <- ggpubr::ggarrange(gg_jpeg, gg_bug,
                                  ncol=2, labels = c("A", "B"), widths = c(1.1, 0.9)) +
        theme(panel.background = element_rect("white", "white", 0, NULL))
ggsave(file.path(figures_dir, "fig-x-bug.png"), gg_bug_panel,  dpi=600, height = 7, width = 15)


###################################
## RRHO
###################################


###################################
## read DET files
hi.IFNa.18h.t.dt <- fread("input/de-florian/02-DE_HI-18h_IFNa_vs_CTL/differential_expression/02-DE_HI-18h_IFNa_vs_CTL-DET_salmon_DESeq2.csv")
endoc.IFNa.24h.t.dt <- fread("input/de-florian/02-DE_endoC-24h_IFNa_vs_CTL/differential_expression/02-DE_endoC-24h_IFNa_vs_CTL-DET_salmon_DESeq2.csv")
endoc.srsf6.kd.t.dt <- fread("input/de-florian/02-DE_endoC_siRNA_SRSF6KD_vs_CTL/differential_expression/02-DE_endoC_siRNA_SRSF6KD_vs_CTL-DET_salmon_DESeq2.csv")

## read DEG files
hi.IFNa.18h.g.dt <- remove_version_nbr(fread("input/de-florian/02-DE_HI-18h_IFNa_vs_CTL/differential_expression/02-DE_HI-18h_IFNa_vs_CTL-DEG_salmon_DESeq2.csv"))
endoc.IFNa.24h.g.dt <- remove_version_nbr(fread("input/de-florian/02-DE_endoC-24h_IFNa_vs_CTL/differential_expression/02-DE_endoC-24h_IFNa_vs_CTL-DEG_salmon_DESeq2.csv"))
endoc.srsf6.kd.g.dt <- remove_version_nbr(fread("input/de-florian/02-DE_endoC_siRNA_SRSF6KD_vs_CTL/differential_expression/02-DE_endoC_siRNA_SRSF6KD_vs_CTL-DEG_salmon_DESeq2.csv"))



## read Salmon tpm
getFC <- function (.shortid, de)
{
    all.refgenes <- gtf$data[shortid %in% .shortid & feature == "gene",]$ensg
    data.table(g = all.refgenes, fc = sapply(all.refgenes, function (ensg) de[Row.names == ensg, 2**log2FoldChange]) )
}

ref.genes <- c("ACTB", "VAPA", "ARF1")

refs <- getFC(ref.genes, hi.IFNa.18h.g.dt)
hi.IFNa.18h.t.salmon <- fread.transcript("input/de-florian/02-DE_HI-18h_IFNa_vs_CTL/differential_expression/02-DE_HI-18h_IFNa_vs_CTL-ALL_TX_salmon.csv",
                                         gtf=gtf,
                                         ref.g = refs$g, correct.fc = refs$fc, mode="Gene")
hi.IFNa.18h.g.salmon <- fread.gene("input/de-florian/02-DE_HI-18h_IFNa_vs_CTL/differential_expression/02-DE_HI-18h_IFNa_vs_CTL-ALL_GENES_salmon.csv")

refs <- getFC(ref.genes, endoc.srsf6.kd.g.dt)
endoc.srsf6.kd.t.salmon <- fread.transcript("input/de-florian/02-DE_endoC_siRNA_SRSF6KD_vs_CTL/differential_expression/02-DE_endoC_siRNA_SRSF6KD_vs_CTL-ALL_TX_salmon.csv",
                                            gtf=gtf, fix.colnames=fix.endoc.kd.colnames,
                                            ref.g = refs$g, correct.fc = refs$fc, mode="Gene")
endoc.srsf6.kd.g.salmon <- fix.endoc.kd.colnames(fread.gene("input/de-florian/02-DE_endoC_siRNA_SRSF6KD_vs_CTL/differential_expression/02-DE_endoC_siRNA_SRSF6KD_vs_CTL-ALL_GENES_salmon.csv"))

refs <- getFC(ref.genes, endoc.IFNa.24h.g.dt)
endoc.IFNa.t.salmon <- fread.transcript("input/de-florian/02-DE_endoC-24h_IFNa_vs_CTL/differential_expression/02-DE_endoC-24h_IFNa_vs_CTL-ALL_TX_salmon.csv",
                                        gtf=gtf, fix.colnames=fix.endoc.IFNa.colnames,
                                        ref.g = refs$g, correct.fc = refs$fc, mode="Gene")
endoc.IFNa.g.salmon <- fix.endoc.IFNa.colnames(fread.gene("input/de-florian/02-DE_endoC-24h_IFNa_vs_CTL/differential_expression/02-DE_endoC-24h_IFNa_vs_CTL-ALL_GENES_salmon.csv"))


###################################
## generate plots

## srsf6 plots: IFNa hi vs EndoC
nbr.tested <- 50 * 96
IFNa.t.de <- cache(paste0("IFNa-t-deps-", nbr.tested),
                   newDE(endoc.IFNa.24h.t.dt, hi.IFNa.18h.t.dt, names=c("EndoC-βH1 IFNα 24h", "Human Islets IFNα 18h")) %>% computeDeps(hi.IFNa.18h.t.salmon, nbr.tested=nbr.tested))
IFNa.t.de$names <- c("EndoC-βH1 IFNα 24h", "Human Islets IFNα 18h")
IFNa.t.de <- IFNa.t.de %>% doRRHO(niter=500)
IFNa.t.de <- hyper_enrichment(IFNa.t.de, MY_PATHWAYS, direction=c("downdown", "upup") )
IFNa.t.de <- hyper_enrichment(IFNa.t.de, MY_PATHWAYS, direction="downdown" )
IFNa.t.de <- hyper_enrichment(IFNa.t.de, MY_PATHWAYS, direction="upup" )

as.data.table(IFNa.t.de$enrichment$downdown$enricher)[startsWith(ID, "UCDR"),]
as.data.table(IFNa.t.de$enrichment$upup$enricher)[startsWith(ID, "UCDR"),]
as.data.table(IFNa.t.de$enrichment$downdown_upup$enricher)[startsWith(ID, "UCDR"),]

## lapply(c("upup", "downdown"),
##        function (dir)
##            fwrite(as.data.table(IFNa.t.de$enrichment[[dir]]$enricher),
##                   file.path(tsv_dir,
##                             paste0("endoc-hi-ifna-", dir, ".tsv")), sep="\t"))

write.enrich <- function(de, direction, file.prefix="a-b-")
{
    lapply(direction,
           function (dir)
               fwrite(as.data.table(de$enrichment[[dir]]$enricher),
                      file.path(tsv_dir,
                                paste0(file.prefix, dir, ".tsv")), sep="\t"))
}

write.enrich(IFNa.t.de, c("upup", "downdown"), file.prefix="endoc-hi-ifna-")


IFNa.g.de <- cache("IFNa-g-deps",
                   newDE(endoc.IFNa.24h.g.dt, hi.IFNa.18h.g.dt, names=c("EndoC-βH1 IFNα 24h", "Human Islets IFNα 18h")) %>% computeDeps(hi.IFNa.18h.g.salmon))
IFNa.g.de <- IFNa.g.de %>% doRRHO(niter=500)
IFNa.g.de <- hyper_enrichment(IFNa.g.de, MY_PATHWAYS, direction=c("downdown", "upup") )
IFNa.g.de <- hyper_enrichment(IFNa.g.de, MY_PATHWAYS, direction="downdown" )
IFNa.g.de <- hyper_enrichment(IFNa.g.de, MY_PATHWAYS, direction="upup" )

as.data.table(IFNa.g.de$enrichment$downdown$enricher)[startsWith(ID, "UCDR"),]
as.data.table(IFNa.g.de$enrichment$upup$enricher)[startsWith(ID, "UCDR"),]
as.data.table(IFNa.g.de$enrichment$downdown_upup$enricher)[startsWith(ID, "UCDR"),]

write.enrich(IFNa.g.de, c("upup", "downdown"), file.prefix="genes-endoc-hi-ifna-")

IFNa.tg <- newTG(IFNa.t.de, IFNa.g.de, endoc.IFNa.t.salmon, hi.IFNa.18h.t.salmon, c("ENST00000244020", "ENST00000483871"))
gg.IFNa.tg <- ggplot(IFNa.tg)
## ggsave(file.path(figures_dir, "fig-x-IFNa.png"), gg.IFNa.tg,  dpi=600, height = 20, width = 14)


###############
## srsf6 plots: endoC srsf6 IFNa vs srsf6 kd
endoc.kd.IFNa.t.de <- cache(paste0("endoc-kd-t-deps-", nbr.tested),
                            newDE(endoc.IFNa.24h.t.dt, endoc.srsf6.kd.t.dt, names=c("EndoC-βH1 IFNα 24h", "EndoC-βH1 SRSF6 kd")) %>% computeDeps(endoc.srsf6.kd.t.salmon, nbr.tested=nbr.tested))
endoc.kd.IFNa.t.de$names <- c("EndoC-βH1 IFNα 24h", "EndoC-βH1 SRSF6 kd")
endoc.kd.IFNa.t.de <- endoc.kd.IFNa.t.de %>% doRRHO(niter=500)
endoc.kd.IFNa.t.de <- hyper_enrichment(endoc.kd.IFNa.t.de, MY_PATHWAYS, direction=c("downdown", "upup") )
endoc.kd.IFNa.t.de <- hyper_enrichment(endoc.kd.IFNa.t.de, MY_PATHWAYS, direction=c("downdown") )
endoc.kd.IFNa.t.de <- hyper_enrichment(endoc.kd.IFNa.t.de, MY_PATHWAYS, direction=c("upup") )
endoc.kd.IFNa.t.de <- hyper_enrichment(endoc.kd.IFNa.t.de, MY_PATHWAYS, direction=c("updown") )
endoc.kd.IFNa.t.de <- hyper_enrichment(endoc.kd.IFNa.t.de, MY_PATHWAYS, direction=c("downup") )

## lapply(c("upup", "downdown", "updown", "downup"),
##        function (dir)
##            fwrite(as.data.table(endoc.kd.IFNa.t.de$enrichment[[dir]]$enricher),
##                   file.path(tsv_dir,
##                             paste0("endoc-kd-ifna-", dir, ".tsv")), sep="\t"))

write.enrich(endoc.kd.IFNa.t.de, c("upup", "downdown", "updown", "downup"), file.prefix="endoc-kd-ifna-")


as.data.table(endoc.kd.IFNa.t.de$enrichment$downdown$enricher)[startsWith(ID, "UCDR"),]
as.data.table(endoc.kd.IFNa.t.de$enrichment$upup$enricher)[startsWith(ID, "UCDR"),]
as.data.table(endoc.kd.IFNa.t.de$enrichment$downdown_upup$enricher)[startsWith(ID, "UCDR"),]

endoc.kd.IFNa.g.de <- cache("endoc-kd-g-deps",
                            newDE(endoc.IFNa.24h.g.dt, endoc.srsf6.kd.g.dt, names=c("EndoC-βH1 IFNα 24h", "EndoC-βH1 SRSF6 kd"))  %>% computeDeps(endoc.srsf6.kd.g.salmon))
endoc.kd.IFNa.g.de <- endoc.kd.IFNa.g.de %>% doRRHO(niter=500)
endoc.kd.IFNa.g.de <- hyper_enrichment(endoc.kd.IFNa.g.de, MY_PATHWAYS, direction=c("downdown", "upup") )
endoc.kd.IFNa.g.de <- hyper_enrichment(endoc.kd.IFNa.g.de, MY_PATHWAYS, direction=c("downdown") )
endoc.kd.IFNa.g.de <- hyper_enrichment(endoc.kd.IFNa.g.de, MY_PATHWAYS, direction=c("upup") )
endoc.kd.IFNa.g.de <- hyper_enrichment(endoc.kd.IFNa.g.de, MY_PATHWAYS, direction=c("updown") )
endoc.kd.IFNa.g.de <- hyper_enrichment(endoc.kd.IFNa.g.de, MY_PATHWAYS, direction=c("downup") )

write.enrich(endoc.kd.IFNa.g.de, c("upup", "downdown", "updown", "downup"), file.prefix="genes-endoc-kd-ifna-")

as.data.table(endoc.kd.IFNa.g.de$enrichment$downdown$enricher)[startsWith(ID, "UCDR"),]
as.data.table(endoc.kd.IFNa.g.de$enrichment$upup$enricher)[startsWith(ID, "UCDR"),]
as.data.table(endoc.kd.IFNa.g.de$enrichment$downdown_upup$enricher)[startsWith(ID, "UCDR"),]

endoc.kd.IFNa.tg <- newTG(endoc.kd.IFNa.t.de, endoc.kd.IFNa.g.de, endoc.IFNa.t.salmon, endoc.srsf6.kd.t.salmon, c("ENST00000244020", "ENST00000483871"))
gg.endoc.kd.IFNa.tg <- ggplot(endoc.kd.IFNa.tg)
## ggsave(file.path(figures_dir, "fig-x-endoc-kd-IFNa.png"), gg.endoc.kd.IFNa.tg,  dpi=600, height = 20, width = 14)

gg_genes <- ggpubr::ggarrange(ggplot(endoc.kd.IFNa.g.de), ggplot(IFNa.g.de),
                              labels = c("A", "B"))  +
    theme(panel.background = element_rect("white", "white", 0, NULL))
ggsave(file.path(figures_dir, "fig-x-redribbon-genes-srfs6.png"), gg_genes,  dpi=600, height = 5, width = 12)


######################
### Figure IFNa: PAPER

gg_flow <- ggplot() +
    geom_label(aes(x = 0:2, y = 0, label = c("EndoC-βH1 SRSF6 kd", "EndoC-βH1 IFNα 24h", "Human Islets IFNα 18h")))  +
    geom_segment(aes(x = c(0.2, 1.2), y = 0, xend = c(0.8, 1.8), yend = 0), size=1,
                     arrow = arrow(length = unit(0.5, "cm"), ends='both')) +
    geom_segment(aes(x = c(0.5, 1.5), y = 0, xend = c(0.5, 1.5), yend = -0.4), size=1,
                 arrow = arrow(length = unit(0.5, "cm"))) +
    xlim(-0.1, 2.1) + ylim(-0.5, 0.2) +
    theme_void() 
gg_flow


mycollapse <- function (genes)
{
    nbr <- 5
    n <- length(genes)
    if (n > nbr)
        return( paste(paste0(genes[1:nbr], collapse=", "),
                       mycollapse(genes[(nbr+1):n]), sep="\n") )
    
    return(paste0(genes[1:n], collapse=", "))
}


d  <- data.frame(x1=c(-1),
                 y1=c(0),
                 x2=c(26),
                 y2=c(2), t=c('a'))
gg_juan <- ggplot() +
    geom_rect(data=d, mapping=aes(xmin=x1, xmax=x2, ymin=y1, ymax=y2, fill=t), color="black", alpha=0.1, show.legend = FALSE) +
    geom_text(aes(x = 6, y = 1.7, label = "SRSF6 REGULATION splicing pathways from Jonàs Juan-Mateu et al. 2018.")) +
    geom_label(aes(x = c(2.2, 8.8, 15.7, 22.6),
                   y = 0.8,
                   label = c(paste("TYPE 2 DIABETES:", mycollapse(TYPE_2_DIABETES), sep=":\n"),
                             paste("POSITIVE REGULATION APOPTOSIS:", mycollapse(POSITIVE_REGULATION_APOPTOSIS), sep=":\n"),
                             paste("JNK SIGNALING CASCADE:", mycollapse(JNK_SIGNALING_CASCADE), sep=":\n"),
                             paste("INSULIN SECRETION:", mycollapse(INSULIN_SECRETION), sep=":\n")
                             )),
               size=3.5, label.r = unit(0.3, 'cm'))  +
    theme_void() 
gg_juan

gg_rrho <- ggpubr::ggarrange(ggplot(endoc.kd.IFNa.t.de),
                             ggplot(IFNa.t.de),
                             ncol=2)

gg_IFNa_Overlap <- ggpubr::ggarrange(gg_flow,
                                     gg_rrho, ncol=1, nrow=2, heights=c(0.1, 1) ) +
    theme(panel.background = element_rect("white", "white", 0, NULL))
ggsave(file.path(figures_dir, "fig-x-IFNa-overlap.png"), gg_IFNa_Overlap,  dpi=600, height = 5, width = 12)


trunc_size <- 52
exclude.down <- "HIV|CANCER|INFLUENZA|LISTERIA|HUNTINGTONS|ALZHEIMER|PROSTATE"
include.down <- "UCDR_|CYTOK|INTER[LF]|LYSO|APOPT|INSULIN|RECEPTOR|STRESS|INFLA|HLA|MHC|UNFOLD|HYPOXIA|STAT[12]?_"
gg_IFNa_Enrich <- ggpubr::ggarrange(gg_juan, ## gg_flow
                                    ggpubr::ggarrange(ggplot(endoc.kd.IFNa.t.de$enrichment$downdown,
                                                             include=include.down,
                                                             exclude=exclude.down,
                                                             text_trunc_length = trunc_size,
                                                             clean = c("(^[^_]*_)|_", " ")) + ggtitle("Down-down") + theme(plot.title = element_text(size=14, hjust=0.5)),
                                                      ggplot(IFNa.t.de$enrichment$downdown,
                                                             include=include.down,
                                                             exclude=exclude.down,
                                                             text_trunc_length = trunc_size,
                                                             clean = c("(^[^_]*_)|_", " ")) + ggtitle("Down-down") + theme(plot.title = element_text(size=14, hjust=0.5)),
                                                      ncol=2),
                                    ncol=1, nrow=2, heights=c(0.18, 1), labels = c("A", "B")) + theme(panel.background = element_rect("white", "white", 0, NULL))
ggsave(file.path(figures_dir, "fig-x-IFNa-enrich-down-down.png"), gg_IFNa_Enrich,  dpi=600, height = 14, width = 15)


## UP-UP

IFNa.inter <- merge(as.data.table(IFNa.t.de$enrichment$upup),
                    as.data.table(endoc.kd.IFNa.t.de$enrichment$upup),
                    by="ID")$ID

bold.up <- paste(IFNa.inter, collapse="|")
include.up <- paste("UCDR_|CYTOK|INTER[LF]|LYSO|APOPT|INSULIN|RECEPTOR|STRESS|INFLA|HLA|MHC|UNFOLD|HYPOXIA|STAT[12]?_", bold.up, sep="|")

gg_IFNa_upup_enrich <- ggpubr::ggarrange(ggplot(endoc.kd.IFNa.t.de$enrichment$upup, text_trunc_length = trunc_size,
                                         clean = c("(^[^_]*_)|_", " ")) + ggtitle("Up-up") + theme(plot.title = element_text(size=14, hjust=0.5)),
                                         ggplot(IFNa.t.de$enrichment$upup, text_trunc_length = trunc_size,
                                                include=include.up,
                                                exclude="HIV",
                                                bold = bold.up,
                                                clean = c("(^[^_]*_)|_", " ")) + ggtitle("Up-up") + theme(plot.title = element_text(size=14, hjust=0.5)),
                                         ncol=2)

gg_IFNa_upup <- ggpubr::ggarrange(gg_flow,
                                  gg_IFNa_upup_enrich,
                                  ncol=1, nrow=2, heights=c(0.04, 1)) + theme(panel.background = element_rect("white", "white", 0, NULL))
ggsave(file.path(figures_dir, "fig-x-IFNa-enrich-up-up.png"), gg_IFNa_upup,  dpi=600, height = 12, width = 15)


