library(RedRibbon)
library(tximport)
library(ggpubr)
library(stringr)

fix.endoc.IFNa.colnames <- function(tx)
{
    nm <- colnames(tx)
    colnames(tx) <- ifelse(grepl( "^(Ctl|IFNa)-", nm),
                           paste(tstrsplit(nm, "-" )[[1]], "endoC", "24h", tstrsplit(nm, "-" )[[3]], sep='-'),
                           nm)
    tx
}

fix.endoc.kd.colnames  <- function (tx)
{
    endoc.srp55.kd.metadata <- fread("input/de-florian/metadata_endoC_siRNA_SRSF6KD.tsv")
    endoc.srp55.kd.metadata[,newid:=paste(condition,"endoC", "siRNA", pairing, sep="-")]
    colnames(tx) <- sapply(colnames(tx),
                           function (name)
                           {
                               newname <- endoc.srp55.kd.metadata[name == id,]
                               if (nrow(newname) == 0)
                                   name
                               else
                                   newname$newid
                           })
    tx
}

fix.hi.cyt.48h.colnames <- function(tx)
{
    nm <- colnames(tx)
    colnames(tx) <- ifelse(grepl( "^(CTL|CYT)", nm),
                           paste(tstrsplit(nm, "[0-9]" )[[1]], "hi", "48h", tstrsplit(nm, "CTL|CYT" )[[2]], sep='-'),
                           nm)

    tx
}

fix.endoc.cyt.48h.colnames <- function(tx)
{
    nm <- colnames(tx)
    colnames(tx) <- ifelse(grepl( "^(Ctl|Cyt)", nm),
                           paste(tstrsplit(nm, "-" )[[1]], "endoC", "48h", tstrsplit(nm, "-" )[[2]], sep='-'),
                           nm)

    tx
}

read.salmon <- function (path, tx2gene)
{
    fn <- list.files(path = path, "quant.sf", recursive=T, full.names=T)
    tximport(fn, type = c("salmon"), tx2gene = tx2gene)
}

correct.ref.g.fc  <- function (dt,
                               ref = data.table(g = c("ENSG00000101558", "ENSG00000075624", "ENSG00000143761"),
                                               fc = rep(1, 3)))
{
    print(ref)
    print(dt)
     for (i in 1:nrow(ref) ) 
    {
        cols <- grep('Ctl', colnames(dt), ignore.case = TRUE)
        ## dt[Row.names == ref$g[i], ..cols] <- dt[Row.names == ref$g[i], ..cols] * ref$fc[i]
        dt[Row.names == ref$g[i], cols] <- dt[Row.names == ref$g[i], ..cols] * ref$fc[i]
    }
    print(dt)
 
    dt
}

## vapa = ENSG00000101558, actb = ENSG00000075624, ARF1 = ENSG00000143761
fread.transcript <- function (fn, gtf,
                              ref.g = c("ENSG00000101558", "ENSG00000075624", "ENSG00000143761"),
                              normalize = TRUE, mode = c("BestTX", "Gene"),
                              correct.fc = NULL, fix.colnames=NULL)
{
    dt <- fread(fn, sep=",")
    if (is.function(fix.colnames))
        dt <- fix.colnames(dt)
    
    if ( normalize )
    {
        if ( "BestTX" == mode[1] )
        {
            transcripts <- merge(dt, gtf$data[ensg %in% ref.g & feature == "transcript", "enst"], by.x="Row.names", by.y="enst")
            
            transcripts$Medians <- rowMedians(as.matrix(transcripts[,6:ncol(transcripts)]))
            ref.t <- transcripts[transcripts[,.I[Medians == max(Medians)], by=gene_name]$V1,]$Row.names
            
            ref.t.index <- which(dt$Row.names %in% ref.t)
            
            dt[, 6:ncol(dt) ] <- as.data.table(rrho_normalize(dt[, 6:ncol(dt) ], ref.t.index))
        }
        else if ( "Gene" == mode[1] )
        {
            dt.groupby <- dt[gene_id %in% ref.g,][,lapply(.SD, sum, na.rm=TRUE), by=gene_id, .SDcols= 6:ncol(dt)]
            colnames(dt.groupby)[1] <- "Row.names"
            dt.rbind <- rbind(dt.groupby, dt, fill=TRUE)
            cn <- colnames(dt)
            dt.rbind <- dt.rbind[, ..cn]

            if (! is.null(correct.fc) )
                dt.rbind <- correct.ref.g.fc(dt.rbind,
                                             ref=data.table(g = ref.g, fc =  correct.fc))
            
            dt.rbind[, 6:ncol(dt.rbind) ] <- as.data.table( rrho_normalize(dt.rbind[, 6:ncol(dt.rbind) ], 1:length(ref.g)) )
            dt  <- dt.rbind[(length(ref.g)+1):nrow(dt.rbind),]
        }
        else
            stop( paste0("Mode is `", mode[1], "' and should one of: `BestTX', `Gene'.") )
    }

    ## dt[,6:ncol(dt)] <- dt.norm
    return(dt)
}



fread.gene <- function (fn, ref.g = c("ENSG00000101558", "ENSG00000075624", "ENSG00000143761"), normalize=T)
{
    dt <- fread(fn, sep =',')
    ref.g.index <- which(dt$Row.names %in% ref.g)
    if ( normalize )
        dt[, 4:ncol(dt) ] <- as.data.table(rrho_normalize(dt[, 4:ncol(dt) ], ref.g.index))
    
    return(dt)
}

## compute_permutation_fc <- function (deg, salmon, nbr.tested = -1, features = "genes")
## {
##     pos  <-  if ("genes" == features || "gene" == features) 4 else 6
##     dt <- merge(deg[,"Row.names"], salmon)[,pos:ncol(salmon)]

##     if (nrow(dt) != nrow(deg) )
##         stop(paste("merge insuffisient number of rows: ", nrow(dt), "!=", nrow(deg) ))

##     prediction <- rrho_expression_prediction(dt, nbr.tested = nbr.tested)

##     fc <- newFC(prediction$index, prediction$beta[,2])
    
##     return( fc )
## }


remove_version_nbr <- function (dt, col="Row.names")
{
    dt[,Row.names := gsub("([^.]*)\\..*$", "\\1", dt$Row.names)]
    
    return(dt)
}

newDE <- function (self, ...) 
{
    UseMethod("newDE")
}


newDE.data.table <- function(a, b, names=c("a", "b"), by="Row.names")
{  
    ab <- merge(a, b, by=by, suffixes = c(".a", ".b"))
    ab  <- ab[! ( is.na(log2FoldChange.a) | is.na(log2FoldChange.b)), ]
    
    structure(
        list(
            ab=ab,
            names=names,
            correlation=NULL),
        class = "DE")
}

newDE.character <- function(a, b, ...)
{
    a.dt <- fread(a)
    b.dt <- fread(b)
    newDE.data.table(a.dt, b.dt, ...)
}

computeDeps <- function (self, ...) 
{
    UseMethod("computeDeps")
}

computeDeps.DE <- function(self, salmon, nbr.tested = -1)
{
    pos  <-  if ( startsWith(salmon$Row.names[1], "ENSG") ) 4 else 6
    dt <- merge(self$ab[,"Row.names"], salmon)[,pos:ncol(salmon)]

    if (nrow(dt) != nrow(self$ab) )
        stop(paste("merge insuffisient number of rows: ", nrow(dt), "!=", nrow(self$ab) ))

    prediction <- rrho_expression_prediction(dt, nbr.tested = nbr.tested)

    self$correlation <- newFC(prediction$index, prediction$beta[,2])
    
    return( self )

}
    
doRRHO <- function (self, niter=96)
{
    self$rrho <- RedRibbon(self$ab$log2FoldChange.a, self$ab$log2FoldChange.b,  enrichment_mode = "hyper-two-tailed", correlation = self$correlation)
    self$quad <- quadrants(self$rrho, whole=FALSE, algorithm = "ea", permutation=TRUE, niter=niter)

    return (self)
}

ggplot.DE <- function (self, ...) 
{
    gg <-  ggplot(self$rrho, quadrants=self$quad, labels=self$names, repel.force = 250, ...)  +
        coord_fixed(ratio = 1, clip = "off")##  +
        ## theme(legend.position = c(0.9, 0.5),
        ##       legend.background = element_rect(fill=alpha('white', 0.2)))
    
    return(gg)
}

hyper_enrichment <- function (self, ...) 
{
    UseMethod("hyper_enrichment")
}

summary.DE <- function(self)
{
    extract.direction <- function (direction = "upup")
    {
        if ( is.null( self$quad[[direction]] ) )
            return(NULL)

        res  <- self$ab[self$quad[[direction]]$positions, .N, by = gene_id.a]
        setorder(res, -"N")
        return(res)
    }

    upup <- extract.direction("upup")
    downdown  <- extract.direction("downdown")
    updown  <- extract.direction("updown")
    downup <- extract.direction("downup")

    return(list(
        upup.counts=upup,
        downdown.counts=downdown,
        updown.counts=updown,
        downup.counts=downup
    )) 
}

protein_coding <- function(genes, .biotype="protein_coding", .gtf=gtf)
{
    merge(data.table(shortid=genes),
          .gtf$data[feature == "gene" & biotype %in% .biotype, "shortid"], all.x=TRUE)$shortid
}


hyper_enrichment.DE <- function (self, pathways, directions="upup", coding=FALSE,
                                 msigdb.fn = "/srv/datasets/public/references/msigdb/v7.0/GMTs/c2.cp.v7.0.symbols.gmt",
                                 set.universe=FALSE)
{
    if (! file.exists(msigdb.fn) )
        msigdb.fn <- "/srv/genomic_data/msigdb/v7.0/GMTs/c2.cp.v7.0.symbols.gmt"
    msigdb <- read.gmt(msigdb.fn)
    
    genes <- unique(unlist(sapply(directions, function (direction) self$ab[self$quad[[direction]]$positions,]$gene_name.a)))

    universe  <- NULL
    if (set.universe)
        universe <- unique(self$ab$gene_name.a)

     
    if (coding)
    {
        if ( ! is.null(universe) )
            universe <- protein_coding(universe)
        genes <- protein_coding(genes)
    }

    ## pathwaydb <- data.table(term=pathway.name, gene=pathway)
    pathwaydb <- do.call(rbind, lapply(names(pathways), function (name) data.table(term=name, gene=pathways[[name]]) ))

    db <- as.data.table(msigdb)
    names(db) <- c("term", "gene")
    TERM2GENE <- rbind(pathwaydb, db)
    enrich <-  enricher(genes, universe=universe, TERM2GENE=TERM2GENE)

    name <- paste0(directions, collapse="_")
    if ( is.null(self$enrichment) )
        self$enrichment <- list()
    
    self$enrichment[[name]]  <- structure(
        list(enricher = enrich),
        class = "enrichment"
    )
    return(self)
}

highlight  <-  function(x, pat, text_trunc_length=60, clean=NULL)
{
    x_trunc <- str_trunc(x, text_trunc_length) 
    if ( ! is.null(clean) )
        x_trunc <- str_replace_all(x_trunc, clean[[1]], clean[[2]])

    x_trunc
}

ggplot.enrichment <- function (self, text_trunc_length = 60, x.col = "GeneRatio",
                               include = "UCDR_", bold="UCDR_", exclude = "EverYthing:;",  ntop = 42, clean=NULL, ...)
{
    df <- as.data.table(self$enricher)
    df[, GeneRatio.str := GeneRatio]
    df[, GeneRatio := sapply(GeneRatio, function (x) eval(parse(text=x)))]
    setorderv(df, x.col)
    df[, Description := ordered(Description, levels = as.character(Description))]

    toinclude <- df[grepl(include, ID), ID]
    toexclude <- setdiff(df[grepl(exclude, ID), ID], toinclude)
    print(toexclude)
    df <- df[! ID %in% toexclude, ]
    if ( x.col == "GeneRatio" )
        top <- setdiff(tail(df$ID, ntop), toinclude)
    else
        top <- setdiff(head(df$ID, ntop), toinclude)
    N <- nrow(df)
    df <- df[ID %in% toinclude | ID %in% top, ]

    ## print(toinclude)
    
    ## df <- df[grepl("UCDR_", ID), ]
    ## print(df)
    ## dotplot(self$enricher, showCategory = 50) +
    ##    scale_y_discrete(labels=function(x) { highlight(str_trunc(x, text_trunc_length), "UCDR_") } ) +
    ##    theme(text = element_text(size=12)) +
    ##    theme(axis.text.y=element_markdown())


    ggplot(df,
           aes_string(x = x.col, y =  "Description")) + 
        geom_point(aes(size = GeneRatio, color = p.adjust)) +
        theme_bw(base_size = 14) +
        scale_colour_gradient(limits=c(0, 0.10), low="red") +
        scale_y_discrete(labels=function(x) { highlight(x, bold, text_trunc_length=text_trunc_length, clean=clean) } ) +
        ylab(NULL) +
        theme(axis.text.y=element_text(face=ifelse(grepl(bold, df$Description), "bold", "plain")), ##  element_markdown(),
              axis.text.x=element_text(angle=60, hjust=1)) +
        guides(size = guide_legend(order = 2), colour = guide_colorbar(order = 1))
}

as.data.table.enrichment <- function(self)
{
    data.table(ID = self$enricher$ID, pvalue = self$enricher$pvalue, p.adjust = self$enricher$p.adjust, geneID = self$enricher$geneID)
}

newTG <- function (transcript, gene,  transcript_expression_a, transcript_expression_b, transcript_list)
{
    structure(
        list(
            t=transcript,
            g=gene,
            te.a=transcript_expression_a,
            te.b=transcript_expression_b,
            tl=transcript_list),
        class = "tg")
}

pair.plot <- function (expression, transcript_name_xxx, title=NULL, max.y = NULL)
{
    ## actinb <- IFNa.salmon[Name == "ENST00000646664", .SD, .SDcols = names(IFNa.salmon) %like% "IFNa-HI-18h|Ctl-HI-18h"]
    ## mean.actinb <- rowMeans(actinb)

    ## mat <- expression[,6:ncol(expression)]
    mat <- expression[Row.names == transcript_name_xxx, 6:ncol(expression)] ## / actinb * mean.actinb
    ## print(mat)
    mat.flat  <- data.table(tpm = unlist(mat[1,]), group = tstrsplit(names(mat), "-" )[[1]], pairs = tstrsplit(names(mat), "-" )[[4]] )

    if ( is.null(max.y ) )
        max.y  <- max(mat) + 10
    ## print(mat.flat)
    
    ggplot(mat.flat, aes(x=group, y=tpm, group=pairs)) +
        geom_point(aes(colour=group), size=2.5, position=position_dodge(width=0.1)) +
        geom_line(size=1, alpha=0.5, position=position_dodge(width=0.1)) +
        ## ggtitle(paste(transcript_name_xxx)) +
        ## xlab('') +
        ylab(paste(transcript_name_xxx, 'TPM')) +
        ggtitle(title) +
        scale_colour_manual(values=c("#009E73", "#D55E00"), guide=FALSE) + 
        theme_bw()  + theme(axis.title.x=element_blank()) + expand_limits(y = c(0, max.y) )

}

ggplot.tg <- function (self, enrichment.direction="downdown", venn.directions=c("downdown", "upup", "downdown_upup"))
{
    max.y <- sapply(self$tl, function (tx) max(self$te.a[Row.names == tx, 6:ncol(self$te.a)], self$te.b[Row.names == tx, 6:ncol(self$te.b)]) ) + 10
    gg.list <- list(
        gg_rrho_t = function () ggplot(self$t) + ggtitle("Transcripts") +
                                theme(plot.title = element_text(size=14), axis.title.x = element_text(size=12), axis.title.y = element_text(size=12)),
        gg_rrho_g = function ()  ggplot(self$g) + ggtitle("Genes")  +
                                 theme(plot.title = element_text(size=14), axis.title.x = element_text(size=12), axis.title.y = element_text(size=12)),
        gg_enrichment = function () {
            ggplot(self$t$enrichment[[enrichment.direction]]) + ggtitle(enrichment.direction) +
                theme(plot.title = element_text(size=14, hjust=0.5))
        },
        gg_venn = function () {
            mclapply(venn.directions,
                     function (direction.str)
                     {
                         ggVennDiagram(list(Transcripts=self$t$enrichment[[direction.str]]$enricher$ID, Genes=self$g$enrichment[[direction.str]]$enricher$ID),
                                       label_alpha = 0.4, label_size = 8) +
                             ggtitle( paste0(direction.str)) +
                             theme(plot.title = element_text(size=14, hjust=0.5), text = element_text(size=14))
                     },
                     mc.cores=96)
        },
        gg_pp_a = function () {
            if (is.null(self$te.a))
                NULL
            else
                mclapply(1:length(self$tl),
                         function (transcript.i) {
                             transcript <- self$tl[transcript.i]
                             pval <- formatC(self$t$ab[Row.names == transcript, pvalue.a],
                                             digits=1, format="e")
                             lfc <- formatC(self$t$ab[Row.names == transcript, log2FoldChange.a],
                                            digits=2, format="f")
                             title <- paste("pval =", pval, "\nlFC =", lfc)
                             pair.plot(self$te.a, transcript, title=title, max.y = max.y[transcript.i])
                         },
                         mc.cores=96)
        },
        gg_pp_b = function () {
            if (is.null(self$te.b))
                NULL
            else
                mclapply(1:length(self$tl),
                         function (transcript.i) {
                             transcript <- self$tl[transcript.i]
                             pval <- formatC(self$t$ab[Row.names == transcript, pvalue.b],
                                             digits=1, format="e")
                             lfc <- formatC(self$t$ab[Row.names == transcript, log2FoldChange.b],
                                            digits=2, format="f")
                             title <- paste("pval =", pval, "\nlFC =", lfc)
                             pair.plot(self$te.b, transcript, title=title, max.y = max.y[transcript.i])
                         },
                         mc.cores=96)
        }
    )


    pdf(file = NULL)
    ggs <- mclapply(gg.list,
                   function(f) f(),
                   mc.cores=96)
    dev.off()

    ## print(ggs$gg_venn)
    ## print(gg)
    gg_rrho <- ggpubr::ggarrange(ggs$gg_rrho_g, ggs$gg_rrho_t, nrow=1, ncol=2)

    
    gg_venn <- do.call(ggpubr::ggarrange, c(ggs$gg_venn, list(ncol = length(ggs$gg_venn), nrow=1) ))

    gg_pp_ab <- list()
    mylen_a <- length(ggs$gg_pp_a) - 1
    if (mylen_a >= 0)
    {
        if (mylen_a > 0)
            ggs$gg_pp_a[1:mylen_a] <- mclapply(ggs$gg_pp_a[1:mylen_a], function(gg) gg + theme(axis.text.x=element_blank()), mc.cores=96)
        gg_pp_a <- do.call(ggpubr::ggarrange, c(ggs$gg_pp_a, list(nrow = length(self$tl), ncol=1, align = "hv") ))
        gg_pp_ab$a <- gg_pp_a
    }
    
    mylen_b <- length(ggs$gg_pp_b) - 1
    if (mylen_b >= 0)
    {    
        if (mylen_a >= 0)
            ggs$gg_pp_b <- mclapply(ggs$gg_pp_b, function(gg) gg + theme(axis.title.y=element_blank()), mc.cores=96)
        if (mylen_b > 0)
            ggs$gg_pp_b[1:mylen_b] <- mclapply(ggs$gg_pp_b[1:mylen_b], function(gg) gg + theme(axis.text.x=element_blank()), mc.cores=96)
        gg_pp_b <- do.call(ggpubr::ggarrange, c(ggs$gg_pp_b, list(nrow = length(self$tl), ncol=1, align = "hv") ))
        gg_pp_ab$b <- gg_pp_b
    }
    gg_pp <- do.call(ggpubr::ggarrange, c(gg_pp_ab, list(ncol=length(gg_pp_ab), align = "hv")))
    
    gg_bottom <- ggpubr::ggarrange(ggs$gg_enrichment,  gg_pp, labels = c("C", "D"), nrow=1, ncol=2, widths=c(10,4))
    
    gg <- ggpubr::ggarrange(gg_rrho, gg_venn, gg_bottom,
                            labels = c("A", "B", ""), heights = c(5, 3, 12),
                            ncol = 1, nrow = 3)
    return(gg)
}
